/*
 * Entagged Audio Tag library
 * Copyright (c) 2003-2005 Raphal Slinckx <raphael@slinckx.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package entagged.audioformats.mp3.util;

import entagged.audioformats.generic.AbstractTagCreator;
import entagged.audioformats.mp3.*;
import entagged.audioformats.*;

import java.nio.*;
import java.util.*;

public class Id3v2TagCreator extends AbstractTagCreator {
	/**  Default Padding size, when the tag is created from scratch (the old cannot be replaced)  */
	public final static int DEFAULT_PADDING = 4000;
	
	public void create(Tag tag, ByteBuffer buf, List fields, int tagSize, int paddingSize) {
		byte[] b;

		//ID3------------------
		buf.put( (byte) 73 ).put( (byte) 68 ).put( (byte) 51 );  //ID3
		//----------------------------------------------------------------------------
		//On genere le nouveau tag
		//Version du tag ID3v2.xx.xx
		buf.put( (byte) 3 );
		buf.put( (byte) 0 );
		//----------------------------------------------------------------------------
		//Ecriture de certains flags (A COMPLETER)
		boolean[] ID3Flags = new boolean[4];

		ID3Flags[0] = false; //unsyncronization is not done
		ID3Flags[1] = false;
		ID3Flags[2] = false;
		ID3Flags[3] = false;
		buf.put( createID3Flags( ID3Flags ) );
		//----------------------------------------------------------------------------
		//On ecrit la taille du nouveau tag ID3
		// Tag length
		int tempSize = ( tagSize - 10 ) + paddingSize;
		
		b = new byte[4];
		for(int i = 0; i < 4; i++)
		    b[i] = (byte) (tempSize >> ((3 - i) * 7) & 0x7f);
		buf.put( b );
		//----------------------------------------------------------------------------
		//Ecriture de l'Header Etendu si le flag est==true (A COMPLETER)
		if ( ID3Flags[1] == true ) {
			//Create the ID3 Extended Header (not used)
			//This may need to be implemented
			//createExtendedHeader();
		}
		//----------------------------------------------------------------------------
		//Ecriture des champs de texte
		Iterator it = fields.iterator();
		while(it.hasNext()) {
			buf.put((byte[]) it.next());
		}
		
		//Fill the rest with \0 (padding)
		for ( int i = 0; i < paddingSize; i++ )
			buf.put( (byte) 0 );
	}

	//Create a byte representing ID3 Flags using the given array
	private byte createID3Flags( boolean[] flag ) {
		byte b = 0;

		if ( flag[0] == true )
			b += 128;
		if ( flag[1] == true )
			b += 64;
		if ( flag[2] == true )
			b += 32;
		if ( flag[3] == true )
			b += 16;
		return b;
	}
	
	protected int getFixedTagLength(Tag tag) {
	    return 10;
	}
	
	protected Tag getCompatibleTag(Tag tag) {
		if(! (tag instanceof Id3v2Tag)) {
		    Id3v2Tag id3Tag = new Id3v2Tag();
		    id3Tag.merge(tag);
		    return id3Tag;
		}		
		return tag;
	}
}
